package dev.shadowsoffire.apotheosis.advancements;

import net.minecraft.world.item.ItemStack;
import net.minecraft.advancements.critereon.AbstractCriterionTriggerInstance;
import java.util.Iterator;
import java.util.stream.Stream;
import java.util.Map;
import net.minecraft.nbt.CompoundTag;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import net.minecraft.nbt.TagParser;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemInstance;
import dev.shadowsoffire.apotheosis.adventure.socket.SocketHelper;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import java.util.Locale;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import java.util.function.Predicate;
import java.util.Objects;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraft.resources.ResourceLocation;
import net.minecraftforge.registries.ForgeRegistries;
import net.minecraft.world.item.enchantment.Enchantment;
import net.minecraft.world.item.SpawnEggItem;
import net.minecraft.advancements.critereon.ItemPredicate;
import net.minecraft.advancements.critereon.MinMaxBounds;
import net.minecraft.util.GsonHelper;
import net.minecraft.advancements.critereon.DeserializationContext;
import net.minecraft.advancements.critereon.ContextAwarePredicate;
import com.google.gson.JsonObject;
import net.minecraft.advancements.critereon.InventoryChangeTrigger;

public class ExtendedInvTrigger extends InventoryChangeTrigger
{
    public InventoryChangeTrigger.TriggerInstance m_7214_(final JsonObject json, final ContextAwarePredicate andPred, final DeserializationContext conditionsParser) {
        final JsonObject slots = GsonHelper.m_13841_(json, "slots", new JsonObject());
        final MinMaxBounds.Ints occupied = MinMaxBounds.Ints.m_55373_(slots.get("occupied"));
        final MinMaxBounds.Ints full = MinMaxBounds.Ints.m_55373_(slots.get("full"));
        final MinMaxBounds.Ints empty = MinMaxBounds.Ints.m_55373_(slots.get("empty"));
        ItemPredicate[] predicate = ItemPredicate.m_45055_(json.get("items"));
        if (json.has("apoth")) {
            predicate = this.deserializeApoth(json.getAsJsonObject("apoth"));
        }
        return new InventoryChangeTrigger.TriggerInstance(andPred, occupied, full, empty, predicate);
    }
    
    ItemPredicate[] deserializeApoth(final JsonObject json) {
        final String type = json.get("type").getAsString();
        if ("spawn_egg".equals(type)) {
            return new ItemPredicate[] { new TrueItemPredicate(s -> s.m_41720_() instanceof SpawnEggItem) };
        }
        if ("enchanted".equals(type)) {
            final Enchantment ench = json.has("enchantment") ? ((Enchantment)ForgeRegistries.ENCHANTMENTS.getValue(new ResourceLocation(json.get("enchantment").getAsString()))) : null;
            final MinMaxBounds.Ints bound = MinMaxBounds.Ints.m_55373_(json.get("level"));
            return new ItemPredicate[] { new TrueItemPredicate(s -> {
                    final Map<Enchantment, Integer> enchMap = EnchantmentHelper.m_44831_(s);
                    if (ench != null) {
                        return bound.m_55390_((int)enchMap.getOrDefault(ench, 0));
                    }
                    else {
                        enchMap.values().stream();
                        Objects.requireNonNull(bound);
                        final Stream<Integer> stream;
                        return stream.anyMatch(bound::m_55390_);
                    }
                }) };
        }
        if ("affix".equals(type)) {
            return new ItemPredicate[] { new TrueItemPredicate(s -> !AffixHelper.getAffixes(s).isEmpty()) };
        }
        if ("rarity".equals(type)) {
            final DynamicHolder<LootRarity> rarity = (DynamicHolder<LootRarity>)RarityRegistry.byLegacyId(json.get("rarity").getAsString().toLowerCase(Locale.ROOT));
            return new ItemPredicate[] { new TrueItemPredicate(s -> !AffixHelper.getAffixes(s).isEmpty() && rarity.isBound() && AffixHelper.getRarity(s) == rarity) };
        }
        if ("gem_rarity".equals(type)) {
            final DynamicHolder<LootRarity> rarity = (DynamicHolder<LootRarity>)RarityRegistry.byLegacyId(json.get("rarity").getAsString().toLowerCase(Locale.ROOT));
            return new ItemPredicate[] { new TrueItemPredicate(s -> s.m_41720_() == Adventure.Items.GEM.get() && rarity.isBound() && AffixHelper.getRarity(s) == rarity) };
        }
        if ("socket".equals(type)) {
            return new ItemPredicate[] { new TrueItemPredicate(s -> SocketHelper.getGems(s).stream().anyMatch(GemInstance::isValid)) };
        }
        if ("nbt".equals(type)) {
            CompoundTag tag;
            try {
                tag = TagParser.m_129359_(GsonHelper.m_13805_(json.get("nbt"), "nbt"));
            }
            catch (final CommandSyntaxException e) {
                throw new RuntimeException((Throwable)e);
            }
            return new ItemPredicate[] { new TrueItemPredicate(s -> {
                    if (!s.m_41782_()) {
                        return false;
                    }
                    else {
                        tag.m_128431_().iterator();
                        final Iterator iterator;
                        while (iterator.hasNext()) {
                            final String key = iterator.next();
                            if (!tag.m_128423_(key).equals(s.m_41783_().m_128423_(key))) {
                                return false;
                            }
                        }
                        return true;
                    }
                }) };
        }
        return new ItemPredicate[0];
    }
    
    private static class TrueItemPredicate extends ItemPredicate
    {
        Predicate<ItemStack> predicate;
        
        TrueItemPredicate(final Predicate<ItemStack> predicate) {
            this.predicate = predicate;
        }
        
        public boolean m_45049_(final ItemStack item) {
            return this.predicate.test(item);
        }
    }
}
